<?xml version="1.0" encoding="UTF-8"?>
<!--
XSLT stylesheet for transformations from xml files described in
http://java.sun.com/products/jfc/tsc/articles/persistence3/
into Java source files.

Alteration prohibited without allowance by author.
See website below for license and details.

This file is part of JinBean Java GUI Builder
Copyright (c) 2005 Jens Becker
Last modified 2005-08-29
jb@intermitto.net
http://jinbean.intermitto.net/
-->
<xsl:stylesheet
	version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:exsl="http://exslt.org/common"
	exclude-result-prefixes = "exsl"
>	
	<xsl:output
		method="xml"
		indent="no"
		omit-xml-declaration="yes"
	/>
	<xsl:param name="className" />
	<xsl:param name="buildVersion" />
	<xsl:param name="buildDate" />
	<xsl:param name="genActionListenerJButton" />
	<xsl:param name="genActionListenerJComboBox" />
	<xsl:param name="genMainMethod" />
	<xsl:template match="/">
<!--
Schreibe eine Map $objectIDTree für alle <object id="target" class="a.b.ClassName" />

$idrefObjects hat Kindelemente <objectMap id="{@id}" /> mit Wert generate-id()
-->
	<xsl:variable name="idrefObjects">
		<xsl:for-each select="//object[@id and @class]">
			<objectMap id="{@id}">
				<xsl:value-of select="generate-id()" />
			</objectMap>
		</xsl:for-each>
	</xsl:variable><xsl:variable name="objectIDTree" select="exsl:node-set($idrefObjects)"/>
<!--
Schreibe eine Map $objectTree für alle <object class="a.b.ClassName" />

$objectCounter hat Kindelemente <objectMap id="{generate-id()}" /> mit folgenden Unterelementen:
	* $objectFQClassName
		ClassName, wenn in java.awt oder javax.swing
		a.b.ClassName sonst
	* $objectVariableName
		className5, wobei 5 die position() ist

-->
	<xsl:variable name="objectCounter">
		<xsl:for-each select="//object[@class]">
			<objectMap id="{generate-id()}">
				<objectFQClassName>
					<xsl:variable name="awt">
						<xsl:call-template name="replace">
							<xsl:with-param name="string" select="@class" />
							<xsl:with-param name="search" select="'java.awt.'" />
						</xsl:call-template>
					</xsl:variable>
					<xsl:variable name="swing">
						<xsl:call-template name="replace">
							<xsl:with-param name="string" select="@class" />
							<xsl:with-param name="search" select="'javax.swing.'" />
						</xsl:call-template>
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="contains($awt, '.') and contains($swing, '.')">
							<xsl:value-of select="@class" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="extractClassName">
								<xsl:with-param name="FQClassName" select="@class"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</objectFQClassName>
				<objectVariableName>
					<xsl:call-template name="extractClassName">
						<xsl:with-param name="FQClassName" select="@class"/>
					</xsl:call-template>
					<xsl:value-of select="position()" />
				</objectVariableName>
			</objectMap>
		</xsl:for-each>
	</xsl:variable><xsl:variable name="objectTree" select="exsl:node-set($objectCounter)"/>
<!--
Starte Output
--><xsl:text>/*
 * Generated with JinBean Java GUI Builder v</xsl:text>
 <xsl:value-of select="$buildVersion" />
 <xsl:text>
 * </xsl:text>
 <xsl:value-of select="$buildDate" /><xsl:text>
 *
 * Copyright (c) 2005 Jens Becker
 * </xsl:text>
 <a href="mailto:jb@intermitto.net">jb@intermitto.net</a>
 <xsl:text>
 *
 * See </xsl:text>
 <a href="http://jinbean.intermitto.net/">http://jinbean.intermitto.net</a>
 <xsl:text>
 * for license and details.
 */

import java.awt.*;
import javax.swing.*;
</xsl:text>
<xsl:if test="$genActionListenerJButton = 'true' or $genActionListenerJComboBox = 'true'">
	<xsl:text>import java.awt.event.*;</xsl:text>
</xsl:if>
<xsl:text>

/**
 * Place JavaDoc information for this class here.
 */
public class </xsl:text><xsl:value-of select="$className" /><xsl:text> extends JFrame</xsl:text>
<xsl:if test="$genActionListenerJButton = 'true' or $genActionListenerJComboBox = 'true'">
	<xsl:text> implements ActionListener</xsl:text>
</xsl:if> {
<!--
Schreibe Attribut-Deklarationen, außer für:
	* JFrame
	* Rectangle
	* Dimension
	* Color
	* Font

Ausgabe:
	private $objectFQClassName $objectVariableName;
-->
	// attributes
<xsl:for-each select="//object[@class]">
	<xsl:if test="not(@field) and @class != 'java.awt.Rectangle' and @class != 'java.awt.Dimension' and @class != 'javax.swing.JFrame' and @class != 'java.awt.Color' and @class != 'java.awt.Font'">
		<xsl:text>	private </xsl:text>
		<xsl:call-template name="objectFQClassName">
			<xsl:with-param name="id">
				<xsl:value-of select="generate-id()" />
			</xsl:with-param>
			<xsl:with-param name="tree">
				<xsl:copy-of select="$objectTree" />
			</xsl:with-param>
		</xsl:call-template>
		<xsl:text> </xsl:text>
		<xsl:call-template name="objectVariableName">
			<xsl:with-param name="id">
				<xsl:value-of select="generate-id()" />
			</xsl:with-param>
			<xsl:with-param name="tree">
				<xsl:copy-of select="$objectTree" />
			</xsl:with-param>
		</xsl:call-template>
		<xsl:text>;&#xA;</xsl:text>
	</xsl:if>
</xsl:for-each>
<!--
Schreibe Konstruktor
-->	/**
	 * Default Constructor. Sets frame preferences and initializes the GUI.
	 */
	public <xsl:value-of select="$className" />() {
		// init contentPane and components
		this.initGUI();

		// frame properties
		this.setVisible(true);
	}

	/**
	 * Create graphical components and display them.
	 */
	private void initGUI() {
<!--
Schreibe Attribut-Initialisierungen, außer für:
	* JFrame
	* Rectangle
	* Dimension
	* Color
	* Font

Ausgabe:
	$objectVariableName = new $objectFQClassName(param1, param2, ..., paramN);
-->		// components
		<xsl:for-each select="//object[@class]">
			<xsl:if test="not(@field) and @class != 'java.awt.Rectangle' and @class != 'java.awt.Dimension' and @class != 'javax.swing.JFrame' and @class != 'java.awt.Color' and @class != 'java.awt.Font'">
				<xsl:call-template name="objectVariableName">
					<xsl:with-param name="id">
						<xsl:value-of select="generate-id()" />
					</xsl:with-param>
					<xsl:with-param name="tree">
						<xsl:copy-of select="$objectTree" />
					</xsl:with-param>
				</xsl:call-template>
				<xsl:text> = new </xsl:text>
				<xsl:call-template name="objectFQClassName">
					<xsl:with-param name="id">
						<xsl:value-of select="generate-id()" />
					</xsl:with-param>
					<xsl:with-param name="tree">
						<xsl:copy-of select="$objectTree" />
					</xsl:with-param>
				</xsl:call-template>
				<xsl:text>(</xsl:text>
				<xsl:for-each select="*[name() != 'void']">
					<!--
					Schreibe Parameter
					-->
					<xsl:call-template name="writeParameter">
						<xsl:with-param name="objectTree">
							<xsl:copy-of select="$objectTree" />
						</xsl:with-param>
						<xsl:with-param name="objectIDTree">
							<xsl:copy-of select="$objectIDTree" />
						</xsl:with-param>
					</xsl:call-template>
					<xsl:if test="position() != last()">
						<xsl:text>, </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:text>)</xsl:text>
				<xsl:text>;&#xA;		</xsl:text>
			</xsl:if>
		</xsl:for-each>
<!--
Schreibe Operationen in der Form
$objectVariableName.operation(param1, param2, ..., paramN);

Ausnahme: wenn parent ebenfalls ein void-Objekt ist, dann
$objectVariableName.getFoo().operation(param1, param2, ..., paramN);

geht alle <void method="" /> oder <void property="" /> durch und schreibt dafür
"parent.this(children);"

-->		<xsl:text>&#xA;		// Properties and methods&#xA;</xsl:text>
		<xsl:for-each select="//object">
			<!-- create variable $prefix with value $objectVariableName or "this" -->
			<xsl:variable name="tempPrefix">
				<xsl:choose>
					<xsl:when test="@class = 'javax.swing.JFrame'">
						<xsl:text>this</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="objectVariableName">
							<xsl:with-param name="id">
								<xsl:value-of select="generate-id()" />
							</xsl:with-param>
							<xsl:with-param name="tree">
								<xsl:copy-of select="$objectTree" />
							</xsl:with-param>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<!-- now go through all void elements of this object -->
			<xsl:for-each select="void">
				<xsl:choose>
					<xsl:when test="name(child::*) = 'void'">
						<!--
						if there are void child elements:
						* add ".getProperty()" to $prefix
						* write operations for all void children
						 -->
						<xsl:variable name="prefix">
							<xsl:value-of select="$tempPrefix" />
							<xsl:text>.get</xsl:text>
							<xsl:call-template name="caseToUpper">
								<xsl:with-param name="word">
									<xsl:value-of select="@property" />
								</xsl:with-param>
							</xsl:call-template>
							<xsl:text>()</xsl:text>
						</xsl:variable>
						<xsl:for-each select="void">
							<!-- write operation -->
							<xsl:call-template name="writeOperation">
								<xsl:with-param name="prefix">
									<xsl:value-of select="$prefix" />
								</xsl:with-param>
								<xsl:with-param name="objectTree">
									<xsl:copy-of select="$objectTree" />
								</xsl:with-param>
								<xsl:with-param name="objectIDTree">
									<xsl:copy-of select="$objectIDTree" />
								</xsl:with-param>
							</xsl:call-template>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<!-- if there are NO void child elements -->
						<!-- write operation -->
						<xsl:call-template name="writeOperation">
							<xsl:with-param name="prefix">
								<xsl:value-of select="$tempPrefix" />
							</xsl:with-param>
							<xsl:with-param name="objectTree">
								<xsl:copy-of select="$objectTree" />
							</xsl:with-param>
							<xsl:with-param name="objectIDTree">
								<xsl:copy-of select="$objectIDTree" />
							</xsl:with-param>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</xsl:for-each>
<!--
Schreibe Registrierungen für ActionListener
-->
		<xsl:if test="$genActionListenerJButton = 'true' or $genActionListenerJComboBox = 'true'">
			<xsl:text>&#xA;		// ActionListener&#xA;		</xsl:text>
			<xsl:for-each select="//object">
				<xsl:if test="@class = 'javax.swing.JButton' and $genActionListenerJButton = 'true'">
					<xsl:call-template name="objectVariableName">
						<xsl:with-param name="id">
							<xsl:value-of select="generate-id()" />
						</xsl:with-param>
						<xsl:with-param name="tree">
							<xsl:copy-of select="$objectTree" />
						</xsl:with-param>
					</xsl:call-template>
					<xsl:text>.addActionListener(this);&#xA;		</xsl:text>
				</xsl:if>
				<xsl:if test="@class = 'javax.swing.JComboBox' and $genActionListenerJComboBox = 'true'">
					<xsl:call-template name="objectVariableName">
						<xsl:with-param name="id">
							<xsl:value-of select="generate-id()" />
						</xsl:with-param>
						<xsl:with-param name="tree">
							<xsl:copy-of select="$objectTree" />
						</xsl:with-param>
					</xsl:call-template>
					<xsl:text>.addActionListener(this);&#xA;		</xsl:text>
				</xsl:if>
			</xsl:for-each>
		</xsl:if>
		<xsl:text>&#xA;	}&#xA;</xsl:text>
		<xsl:if test="$genMainMethod = 'true'">
	/**
	 * The entry point of this class when starting the class directly.
	 *
	 * @param arguments String[] command line arguments
	 */
	public static void main(String[] arguments) {
		new <xsl:value-of select="$className" />();
	}
		</xsl:if>
		<xsl:if test="$genActionListenerJButton = 'true' or $genActionListenerJComboBox = 'true'">
	/**
	 * This operation is called when an ActionEvent occurs.
	 *
	 * @param event ActionEvent
	 */
	public void actionPerformed(ActionEvent event) {
		// to get source component: event.getSource()
	}
		</xsl:if>
}
	</xsl:template>

<!--
Template to write ONE operation.
-->
	<xsl:template name="writeOperation">
		<xsl:param name="prefix" />
		<xsl:param name="objectTree" />
		<xsl:param name="objectIDTree" />
		
		<!-- Write only if the void element has children (= parameter) -->
		<xsl:if test="*">
			<xsl:text>		</xsl:text>
			<xsl:value-of select="$prefix" />
			<xsl:text>.</xsl:text>
			<xsl:choose>
				<!--
				Schreibe
				* @method oder
				* set + caseToUpper(@property)
				-->
				<xsl:when test="@method">
					<xsl:value-of select="@method" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>set</xsl:text>
					<xsl:call-template name="caseToUpper">
						<xsl:with-param name="word">
							<xsl:value-of select="@property" />
						</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:text>(</xsl:text>
			<!--
			Schreibe Parameter
			-->
			<xsl:for-each select="*">
				<xsl:call-template name="writeParameter">
					<xsl:with-param name="objectTree">
						<xsl:copy-of select="$objectTree" />
					</xsl:with-param>
					<xsl:with-param name="objectIDTree">
						<xsl:copy-of select="$objectIDTree" />
					</xsl:with-param>
				</xsl:call-template>
				<xsl:if test="position() != last()">
					<xsl:text>, </xsl:text>
				</xsl:if>
			</xsl:for-each>
			<xsl:text>);&#xA;</xsl:text>
		</xsl:if>
	</xsl:template>

<!--
Template to write ONE parameter.
-->
	<xsl:template name="writeParameter">
		<xsl:param name="objectTree" />
		<xsl:param name="objectIDTree" />
<!--		<xsl:variable name="objectTree" select="exsl:node-set($objectTreeFlat)"/>
		<xsl:variable name="objectIDTree" select="exsl:node-set($objectIDTreeFlat)"/> -->
		<xsl:choose>
			<xsl:when test="name() = 'void' and @property = 'contentPane'">
				<!-- as if <void property="contentPane" /> is an object called contentPane -->
				<xsl:text>contentPane</xsl:text>
			</xsl:when>
			<xsl:when test="name() = 'string'">
				<xsl:text>"</xsl:text>
				<xsl:value-of select="."/>
				<xsl:text>"</xsl:text>
			</xsl:when>
			<xsl:when test="name() = 'char'">
				<xsl:text>'</xsl:text>
				<xsl:value-of select="."/>
				<xsl:text>'</xsl:text>
			</xsl:when>
			<xsl:when test="name() = 'null'">
				<xsl:text>null</xsl:text>
			</xsl:when>
			<xsl:when test="name() = 'object' and @field">
				<xsl:value-of select="@class" />
				<xsl:text>.</xsl:text>
				<xsl:value-of select="@field" />
			</xsl:when>
			<xsl:when test="name() = 'object'">
				<xsl:choose>
					<xsl:when test="@class = 'java.awt.Rectangle' or @class = 'java.awt.Dimension' or @class = 'java.awt.Color' or @class = 'java.awt.Font'">
						<!--
						Schreibe Instanziierung für
						* Rectangle
						* Dimension
						* Color
						* Font
						-->
						<xsl:text>new </xsl:text>
						<xsl:call-template name="objectFQClassName">
							<xsl:with-param name="id">
								<xsl:value-of select="generate-id()" />
							</xsl:with-param>
							<xsl:with-param name="tree">
								<xsl:copy-of select="$objectTree" />
							</xsl:with-param>
						</xsl:call-template>
						<xsl:text>(</xsl:text>
						<xsl:for-each select="*[name() != 'void']">
							<!--
							Schreibe Parameter
							-->
							<xsl:call-template name="writeParameter">
								<xsl:with-param name="objectTree">
									<xsl:copy-of select="$objectTree" />
								</xsl:with-param>
								<xsl:with-param name="objectIDTree">
									<xsl:copy-of select="$objectIDTree" />
								</xsl:with-param>
							</xsl:call-template>
							<xsl:if test="position() != last()">
								<xsl:text>, </xsl:text>
							</xsl:if>
						</xsl:for-each>
						<xsl:text>)</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<!--
						Dem Konstruktor wird ein Objekt übergeben! z.B. new OverlayLayout(jButton1);
						Zwei Fälle:
						* object mit @class: Einbinden über $objectTree
						* object mit @idref: Einbinden über $objectIDTree
						-->
						<xsl:variable name="idParam">
							<xsl:choose>
								<xsl:when test="@class">
									<xsl:value-of select="generate-id()" />
								</xsl:when>
								<xsl:when test="@idref">
									<xsl:call-template name="getID">
										<xsl:with-param name="id">
											<xsl:value-of select="@idref" />
										</xsl:with-param>
										<xsl:with-param name="tree">
											<xsl:copy-of select="$objectIDTree" />
										</xsl:with-param>
									</xsl:call-template>
								</xsl:when>
							</xsl:choose>
						</xsl:variable>
						<xsl:call-template name="objectVariableName">
							<xsl:with-param name="id">
								<xsl:value-of select="$idParam" />
							</xsl:with-param>
							<xsl:with-param name="tree">
								<xsl:copy-of select="$objectTree" />
							</xsl:with-param>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="extractClassName">
		<xsl:param name="FQClassName"/>
		<xsl:choose>
			<xsl:when test="not(contains($FQClassName, '.'))">
				<xsl:value-of select="$FQClassName"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="extractClassName">
					<xsl:with-param name="FQClassName" select="substring-after($FQClassName, '.')"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="objectFQClassName">
		<xsl:param name="id" />
		<xsl:param name="tree" />
		<xsl:variable name="objectTree" select="exsl:node-set($tree)"/>
		<xsl:value-of select="$objectTree/objectMap[@id = $id]/objectFQClassName" />
	</xsl:template>

	<xsl:template name="objectVariableName">
		<xsl:param name="id" />
		<xsl:param name="tree" />
		<xsl:variable name="objectTree" select="exsl:node-set($tree)"/>
		<xsl:call-template name="caseToLower">
			<xsl:with-param name="word">
				<xsl:value-of select="$objectTree/objectMap[@id = $id]/objectVariableName" />
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

<!-- gibt die generate-ID() zu einem object mit @id bzw. @idref zurück -->
	<xsl:template name="getID">
		<xsl:param name="id" />
		<xsl:param name="tree" />
		<xsl:variable name="objectTree" select="exsl:node-set($tree)"/>
		<xsl:value-of select="$objectTree/objectMap[@id = $id]" />
	</xsl:template>

	<xsl:template name="caseToUpper">
		<xsl:param name="word" />
		<xsl:value-of select="concat(translate(substring($word,1,1), 'abcdefghijklmnopqrstuvwxyzäöü', 'ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜ'), substring($word,2,string-length($word)))" />
	</xsl:template>

	<xsl:template name="caseToLower">
		<xsl:param name="word" />
		<xsl:value-of select="concat(translate(substring($word,1,1), 'ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜ', 'abcdefghijklmnopqrstuvwxyzäöü'), substring($word,2,string-length($word)))" />
	</xsl:template>

	<xsl:template name="replace">
		<xsl:param name="string" select="''" />
		<xsl:param name="search" select="/.." />
		<xsl:param name="replace" select="/.." />
		<xsl:choose>
			<xsl:when test="not($string)" />
			<xsl:when test="not($search)">
				<xsl:value-of select="$string" />
			</xsl:when>
			<xsl:otherwise>
				<!-- this converts the search and replace arguments to node sets
				if they are one of the other XPath types -->
				<xsl:variable name="search-nodes-rtf">
					<xsl:copy-of select="$search" />
				</xsl:variable>
				<xsl:variable name="replace-nodes-rtf">
					<xsl:copy-of select="$replace" />
				</xsl:variable>
				<xsl:variable name="replacements-rtf">
					<xsl:for-each select="exsl:node-set($search-nodes-rtf)/node()">
						<xsl:variable name="pos" select="position()" />
						<replace search="{.}">
							<xsl:copy-of select="exsl:node-set($replace-nodes-rtf)/node()[$pos]" />
						</replace>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="sorted-replacements-rtf">
					<xsl:for-each select="exsl:node-set($replacements-rtf)/replace">
						<xsl:sort select="string-length(@search)" data-type="number" order="descending" />
						<xsl:copy-of select="." />
					</xsl:for-each>
				</xsl:variable>
				<xsl:call-template name="_replace">
					<xsl:with-param name="string" select="$string" />
					<xsl:with-param name="replacements" select="exsl:node-set($sorted-replacements-rtf)/replace" />
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="_replace">
		<xsl:param name="string" select="''" />
		<xsl:param name="replacements" select="/.." />
		<xsl:choose>
			<xsl:when test="not($string)" />
			<xsl:when test="not($replacements)">
				<xsl:value-of select="$string" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="replacement" select="$replacements[1]" />
				<xsl:variable name="search" select="$replacement/@search" />
				<xsl:choose>
					<xsl:when test="not(string($search))">
						<xsl:value-of select="substring($string, 1, 1)" />
						<xsl:copy-of select="$replacement/node()" />
						<xsl:call-template name="_replace">
							<xsl:with-param name="string" select="substring($string, 2)" />
							<xsl:with-param name="replacements" select="$replacements" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="contains($string, $search)">
						<xsl:call-template name="_replace">
							<xsl:with-param name="string" select="substring-before($string, $search)" />
							<xsl:with-param name="replacements" select="$replacements[position() > 1]" />
						</xsl:call-template>
						<xsl:copy-of select="$replacement/node()" />
						<xsl:call-template name="_replace">
							<xsl:with-param name="string" select="substring-after($string, $search)" />
							<xsl:with-param name="replacements" select="$replacements" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="_replace">
							<xsl:with-param name="string" select="$string" />
							<xsl:with-param name="replacements" select="$replacements[position() > 1]" />
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>

